@@ -124,6 +124,11 @@ Base::~Base()
     d_ptr = nullptr;
 }
 
+void Base::from7BitString(const char *s, size_t len)
+{
+    from7BitString(QByteArray::fromRawData(s, len));
+}
+
 QByteArray Base::rfc2047Charset() const
 {
     if (d_ptr->encCS.isEmpty()) {
@@ -238,14 +243,19 @@ Structured::~Structured()
     d_ptr = nullptr;
 }
 
-void Structured::from7BitString(const QByteArray &s)
+
+void Structured::from7BitString(const char *s, size_t len)
 {
     Q_D(Structured);
     if (d->encCS.isEmpty()) {
         d->encCS = Content::defaultCharset();
     }
-    const char *cursor = s.constData();
-    parse(cursor, cursor + s.length());
+    parse(s, s + len);
+}
+
+void Structured::from7BitString(const QByteArray &s)
+{
+    from7BitString(s.constData(), s.length());
 }
 
 QString Structured::asUnicodeString() const
@@ -640,7 +650,7 @@ bool Token::parse(const char *&scursor, const char *const send, bool isCRLF)
     }
 
     QPair<const char *, int> maybeToken;
-    if (!parseToken(scursor, send, maybeToken, false /* no 8bit chars */)) {
+    if (!parseToken(scursor, send, maybeToken, ParseTokenNoFlag)) {
         return false;
     }
     d->token = QByteArray(maybeToken.first, maybeToken.second);
@@ -996,6 +1006,13 @@ QVector<QByteArray> Ident::identifiers() const
     return rv;
 }
 
+void Ident::fromIdent(const Ident* ident)
+{
+    d_func()->encCS = ident->d_func()->encCS;
+    d_func()->msgIdList = ident->d_func()->msgIdList;
+    d_func()->cachedIdentifier = ident->d_func()->cachedIdentifier;
+}
+
 void Ident::appendIdentifier(const QByteArray &id)
 {
     Q_D(Ident);
@@ -1160,9 +1177,9 @@ Generic::Generic() : Generics::Unstructured(new GenericPrivate)
 {
 }
 
-Generic::Generic(const char *t) : Generics::Unstructured(new GenericPrivate)
+Generic::Generic(const char *t, int len) : Generics::Unstructured(new GenericPrivate)
 {
-    setType(t);
+    setType(t, len);
 }
 
 Generic::~Generic()
@@ -1190,15 +1207,16 @@ const char *Generic::type() const
     return d_func()->type;
 }
 
-void Generic::setType(const char *type)
+void Generic::setType(const char *type, int len)
 {
     Q_D(Generic);
     if (d->type) {
         delete[] d->type;
     }
     if (type) {
-        d->type = new char[strlen(type) + 1];
-        strcpy(d->type, type);
+        const int l = (len < 0 ? strlen(type) : len) + 1;
+        d->type = new char[l];
+        qstrncpy(d->type, type, l);
     } else {
         d->type = nullptr;
     }
@@ -1662,7 +1680,6 @@ QByteArray ContentType::subType() const {
 void ContentType::setMimeType(const QByteArray & mimeType) {
     Q_D(ContentType);
     d->mimeType = mimeType;
-    Parametrized::clear();
 
     if (isMultipart()) {
         d->category = CCcontainer;
@@ -1794,7 +1811,6 @@ bool ContentType::parse(const char *&scursor, const char *const send,
                         bool isCRLF) {
     Q_D(ContentType);
     // content-type: type "/" subtype *(";" parameter)
-
     clear();
     eatCFWS(scursor, send, isCRLF);
     if (scursor == send) {
@@ -1803,7 +1819,7 @@ bool ContentType::parse(const char *&scursor, const char *const send,
 
     // type
     QPair<const char *, int> maybeMimeType;
-    if (!parseToken(scursor, send, maybeMimeType, false /* no 8Bit */)) {
+    if (!parseToken(scursor, send, maybeMimeType, ParseTokenNoFlag)) {
         return false;
     }
 
@@ -1819,7 +1835,7 @@ bool ContentType::parse(const char *&scursor, const char *const send,
     }
 
     QPair<const char *, int> maybeSubType;
-    if (!parseToken(scursor, send, maybeSubType, false /* no 8bit */)) {
+    if (!parseToken(scursor, send, maybeSubType, ParseTokenNoFlag)) {
         return false;
     }
 
@@ -2080,7 +2096,7 @@ bool ContentDisposition::parse(const char  *&scursor, const char *const send,
     }
 
     QPair<const char *, int> maybeToken;
-    if (!parseToken(scursor, send, maybeToken, false /* no 8Bit */)) {
+    if (!parseToken(scursor, send, maybeToken, ParseTokenNoFlag)) {
         return false;
     }
 
